﻿using Microsoft.AnalysisServices.AdomdClient;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Syncfusion.Reporting.DataExtensions.SSAS
{
    public class SSASDataExtension : Syncfusion.RDL.Data.IDataExtension
    {
        public string CommandText
        {
            get;
            set;
        }

        public string ConnectionString
        {
            get;
            set;
        }

        public int ConnectionTimeout
        {
            get;
            set;
        }

        public string DataProvider
        {
            get;
            set;
        }

        public string Impersonate
        {
            get;
            set;
        }

        public bool IntegratedSecurity
        {
            get;
            set;
        }

        public string Password
        {
            get;
            set;
        }

        public Dictionary<string, object> QueryParameters
        {
            get;
            set;
        }

        public string Username
        {
            get;
            set;
        }

        public Dictionary<string, string> DataFileldsName
        {
            get;
            set;
        }

        public SSASDataExtension()
        {

        }

        public bool ValidateConnection(string connectionString)
        {
            this.ConnectionString = connectionString;
            AdomdConnection connection = new AdomdConnection(GetConnectionString());

            try
            {
                connection.Open();
                return true;
            }
            catch
            {
                return false;
            }
            finally
            {
                connection.Close();
                connection.Dispose();
            }
        }

        public object GetData()
        {
            var connectionString = this.GetConnectionString();
            using (AdomdConnection connection = new AdomdConnection(connectionString))
            {
                try
                {
                    connection.Open();
                    DataTable dataTable = this.GetTable(connection, this.CommandText);
                    return dataTable;
                }
                catch (Exception ex)
                {
                    throw ex;
                }
                finally
                {
                    connection.Close();
                }
            }
        }

        private string GetConnectionString()
        {
            if (this.IntegratedSecurity)
            {
                return this.ConnectionString + ";Integrated Security=SSPI;";
            }
            else if (!string.IsNullOrEmpty(this.Username) && !string.IsNullOrEmpty(this.Password))
            {
                return this.ConnectionString + "; User ID=" + this.Username + "; Password=" + this.Password;
            }

            return this.ConnectionString;
        }

        private DataTable GetTable(AdomdConnection connection, string commandText)
        {
            using (AdomdCommand command = new AdomdCommand(commandText, (AdomdConnection)connection))
            {
                DataTable table = new DataTable();
                this.UpdateParameters(command);

                using (AdomdDataAdapter adoMdDataAdapter = new AdomdDataAdapter(command))
                {
                    adoMdDataAdapter.Fill(table);
                    return table;
                }
            }
        }

        private void UpdateParameters(AdomdCommand command)
        {
            if (this.QueryParameters != null && this.QueryParameters.Count > 0)
            {
                foreach (var queryParameter in this.QueryParameters)
                {
                    (command as AdomdCommand).Parameters.Add(queryParameter.Key, queryParameter.Value);
                }
            }
        }

        public object GetSchemaData()
        {
            return null;
        }
    }
}
